import { Plugin } from 'foundation-sites/js/foundation.core.plugin'
import $ from 'jquery'
import { UserAgent } from './foundation.userAgent'

/**
 * Share module.
 *
 * Additional JS for share links.
 *
 * Currently implementing the Web Share API when share="web-share". *
 * Also calls the UserAgent plugin when the element contains
 * `data-mobile` or `data-platform`.
 *
 * @module foundation.share
 */

class Share extends Plugin {
  /**
   * Creates a new instance.
   * @class
   * @name Share
   * @fires Share#init
   * @param {JQuery} element - jQuery object to show or hide.
   * @param {object} options - Overrides to the default plugin settings.
   */
  _setup (element, options = {}) {
    this.$element = element
    this.className = 'Share'
    // defaults, fallbacks
    this.options = $.extend(true, {}, Share.defaults, this.$element.data(), options)

    this._log('setup', this)

    if (this.options.share === 'web-share') {
      this._init_web_share()
    }

    // Make sure to initialize the UserAgent plugin too.
    if ('mobile' in this.options || 'platform' in this.options) {
      this._log('initialize user agent plugin')
      this.userAgent = new UserAgent(this.$element)
    }
  }

  /**
   * Initialize web share specific JS and event handlers.
   * @function
   * @private
   */
  _init_web_share () {
    if (typeof navigator?.share !== 'function') {
      this._log('not supported')
      return this.hide()
    }
    const data = {
      url: decodeURIComponent(this.options.url || window?.location.href),
      text: decodeURIComponent(this.options.text),
      title: decodeURIComponent(this.options.title)
    }
    if (!navigator.canShare(data)) {
      this._log('not supported for this data', data)
      return this.hide()
    }
    this.$element.on('click.zf.share', (event) => this._handle_web_share(event, data))
    this.show()
    this._log('init', data)
  }

  /**
   * Web share event handler, must be triggered off a UI event (e.g. click).
   * @function
   * @fires Share#shared
   * @param {object} event - The event being handled.
   * @param {object} data - The data to share.
   * @private
   */
  async _handle_web_share (event, data) {
    this._log('open share dialog', data, this)
    event.preventDefault()
    try {
      await navigator.share(data)

      /**
       * Fires when the share promise resolves (depends on OS when that happens).
       * @event Share#shared
       */
      this.$element.trigger('shared.zf.share', [this.$element])

      this._log('shared successfully')
    } catch (err) {
      this._log('sharing failed with', err)
    }
  }

  /**
   * Checks if the hidden class is set.
   * @function
   * @private
   */
  _isHidden () {
    return this.$element.hasClass(this.options.hiddenClass)
  }

  /*
   * Show element.
   * @function
   * @fires Share#shown
   */
  show () {
    if (!this._isHidden()) {
      return
    }
    this._log('show', this)
    this.$element.removeClass(this.options.hiddenClass)

    /**
     * Fires when the target is shown.
     * @event Share#shown
     */
    this.$element.trigger('shown.zf.share')
  }

  /**
   * Hide element.
   * @function
   * @fires Share#hidden
   */
  hide () {
    if (this._isHidden()) {
      return
    }
    this._log('hide', this)
    this.$element.addClass(this.options.hiddenClass)

    /**
     * Fires when the target is hidden.
     * @event Share#hidden
     */
    this.$element.trigger('hidden.zf.share')
  }

  /**
   * Destroys the Share plugin.
   * @function
   * @private
   */
  _destroy () {
    this._log('destroy', this)
    this.$element.off('.zf.share')
    if (this.userAgent) {
      this.userAgent._destroy()
    }
  }

  /**
   * Logs debug output to the console.
   * @function
   * @private
   */
  _log () {
    if (this.options.debug) {
      console.debug('[Share]', `${this.options.share}:`, ...arguments)
    }
  }
}

/**
 * Default settings for plugin.
 */
Share.defaults = {
  /**
   * The url to be shared.
   * @option
   * @type {string|null}
   * @default 'show'
   */
  url: null,

  /**
   * Text to be shared.
   * @option
   * @type {string|null}
   * @default null
   */
  text: null,

  /**
   * A title to be shared (might be ignored by the target).
   * @option
   * @type {string|null}
   * @default null
   */
  title: null,

  /**
   * The class set on the target while it’s hidden.
   *
   * If this is already there on page load, it will hide the target initially.
   * @option
   * @type {string}
   * @default 'hide'
   */
  hiddenClass: 'hide',

  /**
   * Toggle debug output.
   * @option
   * @type {boolean}
   * @default false
   */
  debug: false,
}

export { Share }
